// GPSReader

#define DEBUG 1
#define READ_FILE 1
#include "GPSReader.h"
#include "GPSMessages.h"
#include <app/Application.h>
#include <app/Messenger.h>
#include <app/Message.h>
#include <app/Roster.h>
#include <storage/Directory.h>
#include <storage/File.h>
#include <storage/Path.h>
#include <support/Debug.h>
#include <string.h>

GPSReader::GPSReader(int32 serial_port, BMessenger& dispatcher) 
{
	dest = dispatcher; // set the destination for BMessages
	
#if !READ_FILE
	// try to open the serial port. if we can't, throw an exception
	serial.GetDeviceName(serial_port, portname);
	if (serial.Open(portname) < B_OK) {
    		fprintf(stderr, "Failed to open port: %s\n", portname);
    		throw Error("Could not open the specified serial port");
    }
	// set up the correct port settings
	serial.SetDataRate(B_9600_BPS);
	serial.SetFlowControl(0);
#endif
}

GPSReader::~GPSReader()
{
	serial.Close();
}

int32 GPSReader::thread_func(void *arg)
{
	GPSReader *obj = static_cast<GPSReader*>(arg);
	return (obj->threadFunc());
}

thread_id GPSReader::Go()
{
	my_thread = 
		spawn_thread(thread_func, "GPSReader", B_NORMAL_PRIORITY, this);
	resume_thread(my_thread);
	return my_thread;
}

int32 GPSReader::threadFunc()
{
	// Read GPS data from the serial port one byte at a time.
	// When we get an entire line, send the data to the dispatcher.
	
	char buf[1024];	// buffer for reading in GPS data
	char in; // current byte
	int index(0); // where are we in the buffer?
	BMessage msg;
	
#if READ_FILE
	char absolute_path[256];
	app_info ainfo;
	if (be_app->GetAppInfo(&ainfo) != B_OK)
		exit(1);
	BEntry e(&ainfo.ref);
	BDirectory dir;
	BPath path;
	
	e.GetParent(&dir);
	dir.GetEntry(&e);
	e.GetPath(&path);
	
	const char* pathString = path.Path();
	sprintf(absolute_path, "%s/gps test data", pathString);
	BFile data_file(absolute_path, B_READ_ONLY);
	while (true) {
		// Get a byte until you get a newline
		while (data_file.Read(&in, 1) > 0) {
			if (in == '*') { // we have a whole line
				if (msg.what == GPS_RAW_DATA) {
					buf[index] = 0; // null-terminate the message
					msg.AddString("data", buf); // add the data
					// pack and send the message
					dest.SendMessage(&msg); // send the message
				}
				while (in != '\n')
					data_file.Read(&in, 1);
				// start new message
				msg = BMessage(GPS_RAW_DATA); // make the message
				data_file.Read(buf, 7); // get the message type
				buf[6] = 0;
				msg.AddString("type", buf + 1);
				index = 0;
			} else if (in == ',') {
				buf[index] = 0; // null-terminate the message
				msg.AddString("data", buf); // add the data
				index = 0;
			} else {
				buf[index] = in;
				index++;
			}
			snooze(10000);
		}
		data_file.Seek(0, SEEK_SET);
	}
#else
	while (true) {
		// Get a byte until you get a newline
		while (serial.Read(&in, 1) > 0) {
			if (in == '*') { // we have a whole line
				if (msg.what == GPS_RAW_DATA) {
					buf[index] = 0; // null-terminate the message
					msg.AddString("data", buf); // add the data
					// pack and send the message
					dest.SendMessage(&msg); // send the message
				}
				while (in != '\n')
					serial.Read(&in, 1);
				// start new message
				msg = BMessage(GPS_RAW_DATA); // make the message
				serial.Read(buf, 7); // get the message type
				buf[6] = 0;
				msg.AddString("type", buf + 1);
				index = 0;
			} else if (in == ',') {
				buf[index] = 0; // null-terminate the message
				msg.AddString("data", buf); // add the data
				index = 0;
			} else {
				buf[index] = in;
				index++;
			}
		}
		
		throw Error("Out of data");	
	}
#endif
	DEBUG_ONLY(fprintf(stdout, "There's no more data to read!\n"));
	
	return 0; // shouldn't matter
}

