#define DEBUG 1
#include <support/Debug.h>
#include "GPSServer.h"
#include "GPSMessages.h"

int main() {
	GPSServer s;
	s.Run();
	return 0;
}

GPSServer::GPSServer() : 
	BApplication("application/x-vnd.BUGatUIUC-gps_server") {
	clients = new BList(10);
	DEBUG_ONLY(fprintf(stderr, "Entering the constructor.\n"));
	DEBUG_ONLY(fflush(stderr));
	try {
		reader = new GPSReader(0, be_app_messenger);
	}
	catch (GPSReader::Error e) {
		fprintf(stderr, "Oops. GPSReader threw this:\n%s\n", e.msg);
		fflush(stderr);
		exit(0);
	}
	
	reader_id = reader->Go();
	
	DEBUG_ONLY(fprintf(stderr, "Exiting the constructor.\n"));
	DEBUG_ONLY(fflush(stderr));
}

GPSServer::~GPSServer() {
	kill_thread(reader_id);
	delete reader;
}

// static function that is called for each one of the clients
bool DispatchToClients(void* dest_in, void* msg_in) {
	DEBUG_ONLY(fprintf(stdout, "Calling DispatchToClients\n"));
	DEBUG_ONLY(fflush(stdout));
	BMessage* msg = static_cast<BMessage*>(msg_in);
	BMessenger* dest = static_cast<BMessenger*>(dest_in);
	if (strcmp(msg->FindString("type"), "GPGGA") == 0) {
		msg->PrintToStream();
		dest->SendMessage(msg);
	}
	return false;
}

void GPSServer::MessageReceived(BMessage *msg) {
	switch (msg->what) {
		case GPS_RAW_DATA: {
			// for now, send to everyone in the list
			clients->DoForEach(DispatchToClients, msg);
			break;
		}
		case GPS_SUBSCRIBE: {
			// get the messenger and add it to the list
			DEBUG_ONLY(fprintf(stdout, "Got a subscribe message!\n"));
			DEBUG_ONLY(fflush(stdout));
			BMessenger cli;
			if (msg->FindMessenger("client", &cli) == B_OK) {
				clients->AddItem((new BMessenger(cli)));
			} else {
				DEBUG_ONLY(fprintf(stdout, "Couldn't find a messenger!\n"));
				DEBUG_ONLY(fflush(stdout));
			}
			break;
		}
		case GPS_UNSUBSCRIBE: {
			DEBUG_ONLY(fprintf(stdout, "Got an unsubscribe message!\n"));
			DEBUG_ONLY(fflush(stdout));
			BMessenger cli;
			if (msg->FindMessenger("client", &cli) == B_OK) {
				// find in the client list and remove
				int i;
				BMessenger* current;
				const int count = clients->CountItems();
				for (i = 0; i < count; i++) {
					current = static_cast<BMessenger*>(clients->ItemAt(i));
					if (*current == cli) {
						clients->RemoveItem(i);
						i = count;
					}
				}
			}
			break;
		}
		default: {
			BApplication::MessageReceived(msg);
			break;
		}
	}
}
